﻿using System;
using Microsoft.Xna.Framework;
using Celeste;
using System.Collections.Generic;
using Monocle;
using Celeste.Mod.Entities;

namespace NeedleHelper
{
    // Token: 0x02000745 RID: 1861
    [CustomEntity(new string[]
    {
        "Needle Helper/IWBTGNeedle"
    })]
    [Tracked(false)]
    public class IWBTGNeedle : Entity
    {
        // Token: 0x06002E37 RID: 11831 RVA: 0x0010EC78 File Offset: 0x0010CE78
        // Token: 0x040027F3 RID: 10227
        public string Direction;

        // Token: 0x040027F4 RID: 10228
        private PlayerCollider pc;

        // Token: 0x040027F5 RID: 10229
        private Vector2 imageOffset;

        // Token: 0x040027F8 RID: 10232
        private string spikeType;

        // Token: 0x040027F9 RID: 10233
        public Color EnabledColor;

        // Token: 0x040027FA RID: 10234
        public Color DisabledColor;

        // Token: 0x040027FB RID: 10235
        public bool VisibleWhenDisabled;

        public Sprite sprite;

        public IWBTGNeedle(EntityData data, Vector2 offset) : base(data.Position + offset)
        {
            this.EnabledColor = Color.White;
            this.DisabledColor = Color.White;
            //base..ctor(position);
            base.Depth = -1;
            string direction = data.Attr("direction", "Up");
            this.Direction = direction;
            switch (direction)
            {
                case "Up":
                    //base.Collider = new Hitbox((float)size, 3f, 0f, -3f);
                    this.sprite = new Sprite(GFX.Game, data.Attr("directory", "objects/NeedleHelper") + "/NeedleUp/");
                    base.Collider = new ColliderList(new Collider[]
                    {
                        new Hitbox(16f, 3f, -8f, 5f),
                        new Hitbox(12f, 2f, -6f, 4f),
                        new Hitbox(10f, 2f, -5f, 2f),
                        new Hitbox(8f, 2f, -4f, 0f),
                        new Hitbox(6f, 2f, -3f, -2f),
                        new Hitbox(4f, 2f, -2f, -4f),
                        new Hitbox(2f, 2f, -1f, -6f)
                    });
                    base.Add(new LedgeBlocker(null));
                    break;
                case "Down":
                    this.sprite = new Sprite(GFX.Game, data.Attr("directory", "objects/NeedleHelper") + "/NeedleDown/");
                    base.Collider = new ColliderList(new Collider[]
                    {
                        new Hitbox(16f, 3f, -8f, -8f),
                        new Hitbox(12f, 2f, -6f, -6f),
                        new Hitbox(10f, 2f, -5f, -4f),
                        new Hitbox(8f, 2f, -4f, -2f),
                        new Hitbox(6f, 2f, -3f, 0f),
                        new Hitbox(4f, 2f, -2f, 2f),
                        new Hitbox(2f, 2f, -1f, 4f)
                    });
                    break;
                case "Left":
                    this.sprite = new Sprite(GFX.Game, data.Attr("directory", "objects/NeedleHelper") + "/NeedleLeft/");
                    base.Collider = new ColliderList(new Collider[]
                    {
                        new Hitbox(3f, 16f, 5f, -8f),
                        new Hitbox(2f, 12f, 4f, -6f),
                        new Hitbox(2f, 10f, 2f, -5f),
                        new Hitbox(2f, 8f, 0f, -4f),
                        new Hitbox(2f, 6f, -2f, -3f),
                        new Hitbox(2f, 4f, -4f, -2f),
                        new Hitbox(2f, 2f, -6f, -1f)
                    });
                    base.Add(new LedgeBlocker(null));
                    break;
                case "Right":
                    this.sprite = new Sprite(GFX.Game, data.Attr("directory", "objects/NeedleHelper")+ "/NeedleRight/");
                    base.Collider = new ColliderList(new Collider[]
                    {
                        new Hitbox(3f, 16f, -8f, -8f),
                        new Hitbox(2f, 12f, -6f, -6f),
                        new Hitbox(2f, 10f, -4f, -5f),
                        new Hitbox(2f, 8f, -2f, -4f),
                        new Hitbox(2f, 6f, 0f, -3f),
                        new Hitbox(2f, 4f, 2f, -2f),
                        new Hitbox(2f, 2f, 4f, -1f)
                    });
                    base.Add(new LedgeBlocker(null));
                    break;
            }
            //base.Collider = new Hitbox((float)size, 3f, 0f, -3f);
            this.sprite.Visible = true;
            this.sprite.CenterOrigin();
            //this.sprite.JustifyOrigin(8f,8f);
            this.sprite.Justify = new Vector2?(new Vector2(0.5f, 0.5f));
            this.sprite.AddLoop("loop", "needle", 0.1f, new int[]
            {
                0
            });
            this.sprite.Play("loop", false, false);
            base.Add(this.sprite);
            base.Add(this.pc = new PlayerCollider(new Action<Player>(this.OnCollide), null, null));
            base.Add(new StaticMover
            {
                OnShake = new Action<Vector2>(this.OnShake),
                SolidChecker = new Func<Solid, bool>(this.IsRiding),
                JumpThruChecker = new Func<JumpThru, bool>(this.IsRiding),
                OnEnable = new Action(this.OnEnable),
                OnDisable = new Action(this.OnDisable)
            });
        }

        // Token: 0x06002E38 RID: 11832 RVA: 0x0010EDFD File Offset: 0x0010CFFD
        //public AliceCustomSpike(EntityData data, Vector2 offset, int dir) : this(data.Position + offset, dir)
        //{
        //}

        // Token: 0x06002E39 RID: 11833 RVA: 0x0010EE2C File Offset: 0x0010D02C

        public void SetSpikeColor(Color color)
        {
            foreach (Component component in base.Components)
            {
                Image image = component as Image;
                if (image != null)
                {
                    image.Color = color;
                }
            }
        }

        // Token: 0x06002E3A RID: 11834 RVA: 0x0010EE84 File Offset: 0x0010D084
        public override void Added(Scene scene)
        {
            base.Added(scene);
            AreaData areaData = AreaData.Get(scene);
            this.spikeType = areaData.Spike;
            //this.size / 8
            //Image image = new Image(GFX.Game["objects/booster/outline"]);
            //image.Position = - Vector2.UnitX * 8f - Vector2.UnitY * 8f;
            /*
            switch (this.Direction)
            {
                case 0:
                    image.JustifyOrigin(0.5f, 1f);
                    image.Position = Vector2.UnitX * (0.5f) * 8f + Vector2.UnitY;
                    break;
                case 1:
                    image.JustifyOrigin(0.5f, 0f);
                    image.Position = Vector2.UnitX * (0.5f) * 8f - Vector2.UnitY;
                    break;
                case 2:
                    image.JustifyOrigin(1f, 0.5f);
                    image.Position = Vector2.UnitY * (0.5f) * 8f + Vector2.UnitX;
                    break;
                case 3:
                    image.JustifyOrigin(0f, 0.5f);
                    image.Position = Vector2.UnitY * (0.5f) * 8f - Vector2.UnitX;
                    break;
            }
            */
            //image.CenterOrigin();
            //base.Add(image);
        }

        // Token: 0x06002E3C RID: 11836 RVA: 0x0010F22C File Offset: 0x0010D42C
        private void OnEnable()
        {
            this.Active = (this.Visible = (this.Collidable = true));
            this.SetSpikeColor(this.EnabledColor);
        }

        // Token: 0x06002E3D RID: 11837 RVA: 0x0010F260 File Offset: 0x0010D460
        private void OnDisable()
        {
            this.Active = (this.Collidable = false);
            if (this.VisibleWhenDisabled)
            {
                using (IEnumerator<Component> enumerator = base.Components.GetEnumerator())
                {
                    while (enumerator.MoveNext())
                    {
                        Component component = enumerator.Current;
                        Image image = component as Image;
                        if (image != null)
                        {
                            image.Color = this.DisabledColor;
                        }
                    }
                    return;
                }
            }
            this.Visible = false;
        }

        // Token: 0x06002E3E RID: 11838 RVA: 0x0010F2DC File Offset: 0x0010D4DC
        private void OnShake(Vector2 amount)
        {
            this.imageOffset += amount;
        }

        // Token: 0x06002E3F RID: 11839 RVA: 0x0010F2F0 File Offset: 0x0010D4F0
        public override void Render()
        {
            Vector2 position = this.Position;
            this.Position += this.imageOffset;
            base.Render();
            this.Position = position;
        }

        // Token: 0x06002E40 RID: 11840 RVA: 0x0010F328 File Offset: 0x0010D528
        public void SetOrigins(Vector2 origin)
        {
            foreach (Component component in base.Components)
            {
                Image image = component as Image;
                if (image != null)
                {
                    Vector2 vector = origin - this.Position;
                    image.Origin = image.Origin + vector - image.Position;
                    image.Position = vector;
                }
            }
        }

        // Token: 0x06002E41 RID: 11841 RVA: 0x0010F3A8 File Offset: 0x0010D5A8
        private void OnCollide(Player player)
        {
            switch (this.Direction)
            {
                case "Up":
                    player.Die(new Vector2(0f, 0f), false, true);
                    return;
                    break;
                case "Down":
                    player.Die(new Vector2(0f, 0f), false, true);
                    return;
                    break;
                case "Left":
                    player.Die(new Vector2(0f, 0f), false, true);
                    return;
                    break;
                case "Right":
                    player.Die(new Vector2(0f, 0f), false, true);
                    break;
                default:
                    return;
            }
        }

        // Token: 0x06002E42 RID: 11842 RVA: 0x00100DE1 File Offset: 0x000FEFE1

        // Token: 0x06002E43 RID: 11843 RVA: 0x0010F494 File Offset: 0x0010D694
        private bool IsRiding(Solid solid)
        {
            switch (this.Direction)
            {
                case "Up":
                    return base.CollideCheckOutside(solid, this.Position + Vector2.UnitY);
                case "Down":
                    return base.CollideCheckOutside(solid, this.Position - Vector2.UnitY);
                case "Left":
                    return base.CollideCheckOutside(solid, this.Position + Vector2.UnitX);
                case "Right":
                    return base.CollideCheckOutside(solid, this.Position - Vector2.UnitX);
                default:
                    return false;
            }
        }

        // Token: 0x06002E44 RID: 11844 RVA: 0x0010F520 File Offset: 0x0010D720
        private bool IsRiding(JumpThru jumpThru)
        {
            String direction = this.Direction;
            return direction == "Up" && base.CollideCheck(jumpThru, this.Position + Vector2.UnitY);
        }
    }
}
